#!/usr/local/bin/perl
###########################################################################
#
# Usage:
# $0			-> Uebersicht
# $0?package,section	-> Man-pages fuer package nach sections
#			   geordnet auflisten (default)
# $0?package,alphabet	-> Man-pages fuer package alphabetisch
#			   geordnet auflisten
#
###########################################################################
#
# 19960604 HF * Include into EasyInstall V1.5
# 19960811 HF * Exclude listing of bin, man, lib, libexec, ... as packages
#             * Translate output to english
#
###########################################################################
#
# (c) Copyright 1994, 1995, 1996 Hubert Feyrer.
# Alle Rechte vorbehalten, weitergabe an Dritte ohne schriftliche
# Genehmiging des o.g. Autors ausdruecklich untersagt!
# All rights reserved, distribution of this software without a written
# permission of the author is hereby strictly forbidden!
#
###########################################################################

$pkgbase="/usr/local";


$hostname=`hostname`;

if($ENV{"QUERY_STRING"}){
    local($pkg,$sortby)=split(/,/,$ENV{"QUERY_STRING"});

    $sortby="section" if $sortby eq "";
    $title="Man-pages for package $pkg, sorted by $sortby";
    header();
    listMans($pkg,$sortby);
    footer();
}else{
    $title="Installed Software on $hostname";
    header();
    listPkgs();
    footer();
}


###########################################################################
sub listMans
{
    local($pkg,$sortby)=@_;

    if(!chdir("$pkgbase/$pkg/man")){
	die "warning: can't chdir $pkgbase/$pkg/man\n";
    }

    if($sortby eq "section"){
	local(@mans,$sec,$mans,%mans,$p,$cmd,$url);
	print "<H1>Manual-Pages for $pkg</H1>\n";

	undef @mans;
	foreach $sec ( 1,2,3,4,5,6,7,8,9,l,n ){
	    %sections = ( '1' => 'User-commands',
			 '2' => 'System-calls',
			 '3' => 'Library-calls',
			 '4' => '',
			 '5' => '',
			 '6' => 'Games',
			 '7' => '',
			 '8' => '',
			 '9' => '',
			 'n' => 'New commands',
			 'l' => 'Local Extensions' );
	    chop($mans=`echo ???$sec/*.?`);
	    next if $mans=~/\?\?\?.\/\*.\?/;

	    print "\t<H2>Section $sec: $sections{$sec}</H2>\n";
	    print "\t<UL>\n";

	    undef %mans;
	    foreach $p ( split(/ /,$mans)){
		($cmd)=$p=~/....\/(\S+)\.[01-9ln]/;
		$mans{"$cmd($sec)"}=1;
	    }
	    foreach $p ( sort { lc($a) cmp lc($b) } keys %mans ){
		$url="<A HREF=\"http://rfhs1012.fh.uni-regensburg.de/PoolDoku/bin/man.pl.cgi?$p\">$p</A>";	# insert URL for man2html here!!!
		print "\t\t<LI> $url\n";
	    }

	    print "\t</UL>\n";
	}
    }else{
	local($mans,$p,$sec,$cmd,%mans,@mans,$url);
	print "<H1>Manual-Pages for $pkg</H1>\n";

	chop($mans=`echo */*`);
	foreach $p ( split(/ /,$mans) ){
	    ($sec,$cmd)=$p=~/...([1-9ln])\/(\S+)\.[01-9ln]/;
	    $mans{"$cmd($sec)"}=1;
	}

	print "<UL>\n";
	foreach $p ( sort { lc($a) cmp lc($b) } keys %mans ){
	    $url="<A HREF=\"http://rfhs1012.fh.uni-regensburg.de/PoolDoku/bin/man.pl.cgi?$p\">$p</A>";	# insert URL for man2html here!!!
	    print "\t\t<LI> $url\n";
	}
	print "</UL>\n";
    }
}

###########################################################################
sub listPkgs
{
    local($d,%p,$c,%c,$name_url);

    opendir(D,"$pkgbase")
	or die "Can't readdir '$pkgbase': $!";
    while($d=readdir(D)){
	next if $d eq ".";
	next if $d eq "..";
	next if $d eq "local";
	next if $d eq "install";
	next if $d eq "scripts.div";
	next if $d eq "bin";
	next if $d eq "include";
	next if $d eq "info";
	next if $d eq "lib";
	next if $d eq "libdata";
	next if $d eq "libexec";
	next if $d eq "man";
	next if $d eq "sbin";

	$c="Sonstiges";
	if(open(F,"$pkgbase/$d/install/package")){
	    while(<F>){ last if /^CATEGORY=.+/; }
	    close(F);
	    if(!/^CATEGORY=(.+)/){
		print "warning: no CATEGORY in $pkgbase/$d/install/package!\n";
	    }else{
		$c=$1;
	    }
	}
	if($c=~/['"](.+)['"]/){ $c=$1; }
	$p{$d}=$c;
    }
    closedir(D);

    foreach $c ( values %p ){ $c{$c}=1; }	# Kategorien in %c
    print "<H1>Installed Software-Packages on $hostname</H1>\n";

    print "<UL>\n";
    foreach $c ( sort { lc($a) cmp lc($b) } keys %c ){
	# All categories (Graphics, Utilities, ...)
	print "<LI> <H2>$c</H2>\n";

	print "\t<DL>\n";
	foreach $d ( sort { lc($a) cmp lc($b) } keys %p ){
	    # All programs of one category
	    next if $p{$d} ne $c;

	    # read package-file
	    $pkg_name    = "";
	    $pkg_version = "";
	    $pkg_desc    = "";
	    $name_url    = "";
	    if(open(F,"$pkgbase/$d/install/package")){
		while(<F>){
		    if(/^NAME=(.+)/)                { $pkg_name=$1; }
		    if(/^NAME_URL=['"]?(.+)['"]?$/) { ($name_url=$1)=~s/'$//; }
		    if(/^VERSION=(.+)/)             { $pkg_version=$1; }
		    if(/^DESC[0-9]*="(.+)"\s*$/)    { $pkg_desc.=" $1"; }
		}
                $pkg_name=~s/^["']//; $pkg_version=~s/^["']//;
                $pkg_name=~s/["']$//; $pkg_version=~s/["']$//;
		close(F);
		    
		if($name_url){
		    print ("\t\t<DT> <A HREF=\"$name_url\">$pkg_name ",
			   "(Version: $pkg_version):</A></DT>\n");
		}else{
		    print "\t\t<DT> $pkg_name (Version: $pkg_version):</DT>\n";
		}
		    
  	        $pkg_desc .= ". " if $pkg_desc!~/\.\s*/;
		    
		print "\t\t\t<DD>$pkg_desc";
		if(-d "$pkgbase/$d/man"){
		    print " See ";
		    print "<A HREF=\"installedSW.pl.cgi?$d,section\">";
		    print "Man-Page(s)</A> for more information.";
		}
		print "</DD>\n";
	    }else{
		print "\t\t<DT>$d</DT>\n";
		if(-d "$pkgbase/$d/man"){
		    print "<DD>See ";
		    print "<A HREF=\"installedSW.pl.cgi?$d,section\">";
		    print "Man-Page(s)</A> for more information.</DD>\n";
		}
	    }
	}
	print "\t</DL>\n";
    }
    print "</UL>\n";
}

###########################################################################
sub header
{
    print "Content-type: text/html\n\n";

    print "<HTML>\n";
    print "<HEAD>\n";
    print "<TITLE>$title</TITLE>\n";
    print "</HEAD>\n";
    print "<BODY>\n";
    if($debug){

	print "<HR>\n";
	
	{
	    # print ARGC
	    local($i);	 
	    
	    for($i=0;$i<=$#ARGC;$i++){
		print "ARGV[$i]='$ARGV[$i]'<BR>\n";
	    }		 
	    print "<P>\n";	
	}
	
	{
	    # print ENV 
	    local($var);	 
	    
	    foreach $var ( sort keys %ENV ){
		print "ENV: $var=$ENV{$var}<BR>\n";
	    }		 
	    print "<P>\n";
	}

	print "<HR>\n";
    }
}

###########################################################################
sub footer
{
    local($today,$d,$m,$y,$date);

    print "<HR>\n";

    ($d,$m,$y)=(localtime)[3,4,5];
    $date=sprintf("%02d.%02d.%02d",$d,$m+1,$y);
    print ("<A HREF=\"http://rfhs1012.fh.uni-regensburg.de/~feyrer/\">",
	   "hubert.feyrer\@rz.uni-regensburg.de</A>, $date\n");

    print "</BODY>\n";
    print "</HTML>\n";
}
